Feature: upgrade frontend
  As a Tails user
  I want to be guided through upgraded my Tails system, if needed

  Background:
    Given a usable temporary directory
    And a trusted OpenPGP signing key pair
    And a HTTPS server with a valid SSL certificate

  Scenario: manually installed USB: no upgrade is available
    Given Tails is running from a manually installed USB thumb drive
    And no upgrade is available
    And a Live filesystem
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be told "The system is up-to-date"

  Scenario: manually installed USB: no incremental upgrade is available, but a full upgrade is
    Given Tails is running from a manually installed USB thumb drive
    And no incremental upgrade is available, but a full upgrade is
    And a Live filesystem
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be proposed to install this upgrade
    And the web page about the new release should be opened

  Scenario: manually installed USB: both incremental and full upgrades are available
    Given Tails is running from a manually installed USB thumb drive
    And both incremental and full upgrades are available
    And a Live filesystem
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be proposed to install this upgrade
    And the web page about the new release should be opened

  Scenario: DVD: no upgrade is available
    Given Tails is running from a DVD
    And no upgrade is available
    And a Live filesystem
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be told "The system is up-to-date"

  Scenario: DVD: no incremental upgrade is available, but a full upgrade is
    Given Tails is running from a DVD
    And no incremental upgrade is available, but a full upgrade is
    And a Live filesystem
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be proposed to install this upgrade
    And the web page about the new release should be opened

  Scenario: DVD: both incremental and full upgrades are available
    Given Tails is running from a DVD
    And both incremental and full upgrades are available
    And a Live filesystem
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be proposed to install this upgrade
    And the web page about the new release should be opened

  Scenario: USB produced by our installer: no upgrade available
    Given Tails is running from a USB thumb drive
    And no upgrade is available
    When I run tails-upgrade-frontend
    Then it should succeed
    And I should be told "The system is up-to-date"

  Scenario: USB produced by our installer: cannot determine whether an upgrade is available
    Given Tails is running from a USB thumb drive
    And it is not known whether an upgrade is available
    When I run tails-upgrade-frontend
    Then it should fail
    And I should be pointed to the documentation about upgrade-description file retrieval error

  Scenario: USB produced by our installer: installing an incremental upgrade
    Given Tails is running from a USB thumb drive
    And both incremental and full upgrades are available
    And a Live filesystem
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be proposed to install this upgrade
    And I should be told "Downloading the upgrade"
    And I should be asked to wait
    And the network should be shutdown
    And I should be told "The system is now being upgraded"
    And I should be asked to wait
    And the downloaded IUK should be installed
    And I should be proposed to restart the system
    And the system should be restarted
    And the test environment should be cleaned up

  Scenario: USB produced by our installer: no incremental upgrade is available, but a full upgrade is
    Given Tails is running from a USB thumb drive
    And no incremental upgrade is available, but a full upgrade is
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be proposed to install this upgrade
    And the web page about the new release should be opened

  Scenario: USB produced by our installer: both incremental and full upgrades are available, but a target file does not exist
    Given Tails is running from a USB thumb drive
    And both incremental and full upgrades are available
    And a target file does not exist
    When I run tails-upgrade-frontend in batch mode
    Then it should fail
    And I should be proposed to install this upgrade
    And I should be told "Downloading the upgrade"
    And I should be told "Failed to download"
    And I should be told "request failed"
    And I should be pointed to the documentation about target file retrieval error

  Scenario: USB produced by our installer: both incremental and full upgrades are available, but a target file is corrupted
    Given Tails is running from a USB thumb drive
    And both incremental and full upgrades are available
    And a target file is corrupted
    When I run tails-upgrade-frontend in batch mode
    Then it should fail
    And I should be proposed to install this upgrade
    And I should be told "Downloading the upgrade"
    And I should be told "Failed to download"
    And I should be told "was downloaded but its size"
    And I should be pointed to the documentation about target file retrieval error

  Scenario: USB produced by our installer: not enough free memory
    Given Tails is running from a USB thumb drive
    And both incremental and full upgrades are available
    And a Live filesystem
    And the system has not enough free memory to install this incremental upgrade
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be proposed to install this upgrade
    And I should be told "requires .+ of free memory, but only .+ is available"
    And the web page about the new release should be opened

  Scenario: USB produced by our installer: not enough free space on the system partition
    Given Tails is running from a USB thumb drive
    And both incremental and full upgrades are available
    And a Live filesystem
    And the system partition has not enough free space to install this incremental upgrade
    When I run tails-upgrade-frontend in batch mode
    Then it should succeed
    And I should be proposed to install this upgrade
    And I should be told "requires .+ of free space on Tails system partition"
    And the web page about the new release should be opened
